/*
 * (C) Copyright 2002
 * Kyle Harris, Nexus Technologies, Inc. kharris@nexus-tech.net
 *
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include "armboot.h"
#include "pxa-regs.h"

/* ------------------------------------------------------------------------- */


/*
 * Miscelaneous platform dependent initialisations
 */

int board_init(bd_t *bd)
{
    /* memory and cpu-speed are setup before relocation */
    /* so we do _nothing_ here */

    /* arch number of Lubbock-Board */
    bd->bi_arch_number = 89;

    /* adress of boot parameters */
    bd->bi_boot_params = 0xa0000100;

    return 1;
}

int dram_init(bd_t *bd)
{
    unsigned long size, mdcnfg = MDCNFG;

    switch( mdcnfg & 0x60)
    {
	case 0x20:	// ROWADDR_12b
	    size = 0x2000000; break;
    	case 0x40:	// ROWADDR_13b
	    size = 0x4000000; break;
	default:
	    size = 0;
    }

    if( mdcnfg & MDCNFG_DWID0)
	    size = (size >> 1);
    
    bd->bi_dram[0].start = PHYS_SDRAM_1;
    bd->bi_dram[0].size  = size;
    
    bd->bi_dram[1].start = PHYS_SDRAM_2;
    bd->bi_dram[1].size  = 0;
    bd->bi_dram[2].start = PHYS_SDRAM_3;
    bd->bi_dram[2].size  = 0;
    bd->bi_dram[3].start = PHYS_SDRAM_4;
    bd->bi_dram[3].size  = 0;
   
    return bd->bi_dram[0].size;
}

/* Crystal Frequency to Memory Frequency Multiplier (L) */
static unsigned char L_clk_mult[32] = { 0, 27, 32, 36, 40, 45, 0, };

/* Memory Frequency to Run Mode Frequency Multiplier (M) */
static unsigned char M_clk_mult[4] = { 0, 1, 2, 0 };

/* Run Mode Frequency to Turbo Mode Frequency Multiplier (N) */
/* Note: we store the value N * 2 here. */
static unsigned char N2_clk_mult[8] = { 0, 0, 2, 3, 4, 0, 6, 0 };

static void pxa_display_clocks(void)
{
	unsigned long cccr, turbo;
	unsigned int l, L, m, M, n2, N;

	cccr = CCCR;
	asm( "mrc\tp14, 0, %0, c6, c0, 0" : "=r" (turbo) );

	l  =  L_clk_mult[(cccr >> 0) & 0x1f];
	m  =  M_clk_mult[(cccr >> 5) & 0x03];
	n2 = N2_clk_mult[(cccr >> 7) & 0x07];

	L = l * CFG_HZ;
	M = m * L;
	N = n2 * M / 2;

	L += 5000;
	printf("Clock: Mem=%d.%02dMHz (*%d)", L / 1000000, (L % 1000000) / 10000, l );
	
	M += 5000;
	printf(", Run=%d.%02dMHz (*%d)", M / 1000000, (M % 1000000) / 10000, m );
	
	N += 5000;
	printf(", Turbo=%d.%02dMHz (*%d.%d,%sactive)\n",
		N / 1000000, (N % 1000000) / 10000, n2 / 2, (n2 % 2) * 5, (turbo & 1) ? "" : "in" );
}

static const char* pxa210 = "PXA210";
static const char* pxa250 = "PXA250";
static const char* pxa255 = "PXA255";
static const char* unknown = "Unknown";

static const char* revision_id[8] = { "A0","A1","B0","B1","B2","C0","A0","??" };

void board_print_info(bd_t *bd)
{
	ulong processor_id;

	asm("mrc p15,0,%0,c0,c0" : "=r" (processor_id));

	printf("CPU: ");

	if( (processor_id & 0xfffff000) == 0x69052000)
	{
		const char* proc = unknown;

		switch(processor_id & 0xff0)
		{
			case 0x920:	// pxa210
				proc = pxa210; break;
			case 0x100:	// pxa250
			case 0x900:
				proc = pxa250; break;
			case 0xd00:	// pxa250 & pxa255
				proc = pxa250;
				if( (processor_id & 0x0f) > 5)
					proc = pxa255;
			default:
				break;
		}
		printf("Intel XScale-%s (ARM 5TE) revision %s\n", proc,
			((processor_id&0x07)<8)? revision_id[processor_id&0x07]:"??");

		pxa_display_clocks();
	}
	else
		printf("%s\n",unknown);
}

