/*
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Boot support
 */
#include <armboot.h>
#include <command.h>
#include <cmd_net.h>
#include <net.h>

#if (CONFIG_COMMANDS & CFG_CMD_NET)

# if (CONFIG_COMMANDS & CFG_CMD_AUTOSCRIPT)
# include <cmd_autoscript.h>
# endif

extern int do_bootm (cmd_tbl_t *, bd_t *, int, int, char *[]);

static int netboot_common (int, cmd_tbl_t *, bd_t *, int , char *[]);

int do_bootp (cmd_tbl_t *cmdtp, bd_t *bd, int flag, int argc, char *argv[])
{
	return netboot_common (BOOTP, cmdtp, bd, argc, argv);
}

int do_tftpb (cmd_tbl_t *cmdtp, bd_t *bd, int flag, int argc, char *argv[])
{
	return netboot_common (TFTP, cmdtp, bd, argc, argv);
}

int do_rarpb (cmd_tbl_t *cmdtp, bd_t *bd, int flag, int argc, char *argv[])
{
	return netboot_common (RARP, cmdtp, bd, argc, argv);
}

int do_ping (cmd_tbl_t *cmdtp, bd_t *bd, int flag, int argc, char *argv[])
{
	int reg;
	char *s, *e;
        
	if (argc < 2) {
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}
	
	NetServerIP = 0;
	s = argv[1];

	for (reg=0; reg<4; ++reg)
	{
		ulong val = s ? simple_strtoul(s, &e, 10) : 0;
		NetServerIP <<= 8;
		NetServerIP |= (val & 0xFF);
		if (s) s = (*e) ? e+1 : e;
	}

	if (NetLoop(bd, PING) == 0)
		return 0;

	return 1;
}

#if (CONFIG_COMMANDS & CFG_CMD_DHCP)
int do_dhcp (cmd_tbl_t *cmdtp, bd_t *bd, int flag, int argc, char *argv[])
{
	return netboot_common(DHCP, cmdtp, bd, argc, argv);
}
#endif	/* CFG_CMD_DHCP */

#if defined(CONFIG_DRIVER_AX88796) && (CONFIG_COMMANDS & CFG_CMD_MII)

#define MII_MDO_MASK	0x08
#define MII_MDC_MASK	0x01

#define MII_CTRL (*((volatile u32 *)(E8390_BASE+0x28)))

static void mii_write32( unsigned data, int count)
{
	while( count--)
	{
		// output data bit
		MII_CTRL = (data & 0x80000000)? MII_MDO_MASK:0;
		udelay(5);

		// clock pulse
		MII_CTRL = MII_MDC_MASK|((data & 0x80000000)? MII_MDO_MASK:0);
		udelay(5);

		data <<= 1;
	}
}

#define MII_MDI_MASK	0x04
#define MII_DIR_MASK	0x02

static unsigned mii_read32( int count)
{
	unsigned value = 0;
	
	while( count--)
	{
		MII_CTRL = MII_DIR_MASK;
		udelay(5);

		MII_CTRL = MII_DIR_MASK|MII_MDC_MASK;
		udelay(5);

		value <<= 1;
		
		if( MII_CTRL & MII_MDI_MASK)
			value |= 1;
	}
	return value;
}

#define MII_READ	0x60000000
#define MII_WRITE	0x50020000

int do_mii (cmd_tbl_t *cmdtp, bd_t *bd, int flag, int argc, char *argv[])
{
	if( argc > 2 && argv[0][3] == '.')
	{
		unsigned value = 0;
		uchar phy_addr = simple_strtoul(argv[1], NULL, 16);
		uchar phy_reg = simple_strtoul(argv[2], NULL, 16);
		
		if (argv[0][4] == 'w' && argc > 3)
		{
			// do mii write
			value = simple_strtoul(argv[3], NULL, 16);
			mii_write32( MII_WRITE | ((phy_addr&0x1f)<<23) | ((phy_reg&0x1f)<<18), 16);
			mii_write32( (value<<16), 16);	// 16bits DATA
			mii_read32(1);	// read Z

			return 0;
		}
		
		if( argv[0][4] == 'r')
		{
			// do mii read
			mii_write32( MII_READ | ((phy_addr&0x1f)<<23) | ((phy_reg&0x1f)<<18), 14);
			value = mii_read32( 19);	// read TA + 16bits DATA + Z
			printf("mii[%d]: value=0x%04x\n", phy_reg, value>>2);
			
			return 0;
		}
	}

	printf ("Usage:\n%s\n", cmdtp->usage);
	return 1;
}
#endif  /* CFG_CMD_MII */

static void netboot_update_env(void)
{
    char tmp[16] ;

    if (NetOurGatewayIP) {
	ip_to_string (NetOurGatewayIP, tmp);
	setenv(Net_bd, "gatewayip", tmp);
    }

    if (NetOurSubnetMask) {
	ip_to_string (NetOurSubnetMask, tmp);
	setenv(Net_bd, "netmask", tmp);
    }

    if (NetOurHostName[0])
	setenv(Net_bd, "hostname", NetOurHostName);

    if (NetOurRootPath[0])
	setenv(Net_bd, "rootpath", NetOurRootPath);

    if (NetOurIP) {
	ip_to_string (NetOurIP, tmp);
	setenv(Net_bd, "ipaddr", tmp);
    }

    if (NetServerIP) {
	ip_to_string (NetServerIP, tmp);
	setenv(Net_bd, "serverip", tmp);
    }

    if (NetOurDNSIP) {
	ip_to_string (NetOurDNSIP, tmp);
	setenv(Net_bd, "dnsip", tmp);
    }
}

static int
netboot_common (int proto, cmd_tbl_t *cmdtp, bd_t *bd, int argc, char *argv[])
{
	int rc = 0;
   
	switch (argc) {
	case 1:
		break;

	case 2:	/* only one arg - accept two forms:
		 * just load address, or just boot file name.
		 * The latter form must be written "filename" here.
		 */
		if (argv[1][0] == '"') {	/* just boot filename */
			copy_filename (BootFile, argv[1], sizeof(BootFile));
		} else {			/* load address	*/
			load_addr = simple_strtoul(argv[1], NULL, 16);
		}
		break;

	case 3:	load_addr = simple_strtoul(argv[1], NULL, 16);
		copy_filename (BootFile, argv[2], sizeof(BootFile));

		break;

	default: printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	if (NetLoop(bd, proto) == 0)
		return 0;

	/* NetLoop ok, update environment */
	netboot_update_env();

#ifdef CONFIG_AUTOSCRIPT
{
	char *s;	
	/* Loading ok, check if we should attempt an auto-start */
	if (((s = getenv(Net_bd, "autostart")) != NULL) && (strcmp(s,"yes") == 0)) {
		char *local_args[2];
		local_args[0] = argv[0];
		local_args[1] = NULL;

		printf ("Automatic boot of image at addr 0x%08lX ...\n",
			load_addr);

		rc = do_bootm (cmdtp, bd, 0, 1, local_args);
	}

	if (rc == 0 && 
	    ((s = getenv(Net_bd, "autoscript")) != NULL) && (strcmp(s,"yes") == 0)) {
		printf("Running autoscript at addr 0x%08lX ...\n", load_addr);
		rc = autoscript (bd, load_addr);
	}
}
#endif

	return rc;
}

#endif	/* CFG_CMD_NET */
