// intel hex record file format parser

#include <armboot.h>
#include <ihex_record.h>

static uchar _ahtoc( char bcd)
{
	return (((bcd & 0x40)? (bcd-0x37):bcd) & 0x0f);
}

#define _read_char(from)	(_ahtoc(*from++) << 4 | _ahtoc(*from++))

int ihex_decode( const char** pdata, INTEL_HEX_RECORD* ihex)
{
	const char* data = *pdata;
	int i, crc;

	// kill whitespace
	while( *data < 0x1f) data++;

	// read intel hex record header
	if( *data++ != ':')
	{
		printf( "Intel HEX format error: colon expected\n");
		return -1;
	}

	// read hex record length
	crc = ihex->length = _read_char(data);

	if( ihex->length > MAX_INTEL_HEX_RECORD_LENGTH)
	{
		printf( "Intel HEX format error: invalid size\n");
		return -1;
	}

	if( ihex->length > 0)
	{
		uchar* pchar = (uchar*) &(ihex->address);

		// readin address & type
		*(pchar+1) = _read_char(data);
		*pchar = _read_char(data);
		crc += ( *pchar + *(pchar+1));

		ihex->type = _read_char(data);
		crc += ihex->type;
		
		pchar = ihex->data;
			
		// scan data
		for( i = 0; i < ihex->length; i++, pchar++)
		{
			*pchar = _read_char(data);
			crc += *pchar;
		}

		// check crc
		crc += _read_char(data);

		if( crc & 0xff)
		{
			printf( "Intel HEX format error: crc failed\n");
			return -1;
		}

		*pdata = data;
	}
	return ihex->length;
}

