/*
 * (C) Copyright 2002
 * Kyle Harris, Nexus Technologies, Inc. kharris@nexus-tech.net
 *
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * Configuation settings for the LUBBOCK board.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_H
#define __CONFIG_H

/*
 * If we are developing, we might want to start armboot from ram
 * so we MUST NOT initialize critical regs like mem-timing ...
 */
#define CONFIG_INIT_CRITICAL            /* undef for developing */
//#undef CONFIG_INIT_CRITICAL
/*
 * High Level Configuration Options
 * (easy to change)
 */
#define CONFIG_PXA250           1       /* This is an PXA250 CPU    */
#define CONFIG_LUBBOCK          1       /* on an LUBBOCK Board      */

/*
 * Configyure target platform
 * (at least one must be selected)
 */
//#define CONFIG_TRITON		/* KARO electronics dimm module */
//#define CONFIG_CSB226		/* COGENT single board */
#define CONFIG_DIMMPC		/* Voipac dimmPC module baseboard */
//#define CONFIG_EM319		/* EMtest board */

#undef CONFIG_USE_IRQ                   /* we don't need IRQ/FIQ stuff */

/*
 * Size of malloc() pool
 */
#define CONFIG_MALLOC_SIZE      (CFG_ENV_SIZE + 512*1024)

/*
 * Hardware drivers
 */

#ifdef CONFIG_TRITON
// Eth controller for TRITON Startr Kit
#define CONFIG_DRIVER_SMC91111	1
#define CONFIG_SMC91111_BASE 0x21800300

#elif defined(CONFIG_CSB226)
// Eth controller for COGENT CSB226
#define CONFIG_DRIVER_CS8900	1	/* we have a CS8900 on-board */
#define CS8900_BASE             0x08000300
#define CS8900_BUS32            1

#elif defined(CONFIG_DIMMPC)
// Eth controller for Voipac dimmPC module
#define CONFIG_DRIVER_AX88796	1	/* we have a 8390 chipset driver on board */
#define E8390_BASE              0x08000400
#define CONFIG_COMMANDS         (CONFIG_CMD_DFL | CFG_CMD_MII)
//#define CONFIG_COMMANDS      (CFG_CMD_MEMORY|CFG_CMD_WTAGS|CFG_CMD_FLASH|CFG_CMD_NET|CFG_CMD_DHCP)
#elif defined(CONFIG_EM319)
// Eth controller for EMtest EM319 board 
#define CONFIG_DRIVER_CS8900	1	/* we have a CS8900 chipset driver on board */
#define CS8900_BASE             0x04000300
#define CS8900_BUS16            1
#endif

/*
 * select serial console configuration
 */
#if defined(CONFIG_EM319)
#define CONFIG_STUART          1	/* EMtest use STUART */
#else
#define CONFIG_FFUART          1       /* we use FFUART on LUBBOCK */
#endif

/* allow to overwrite serial and ethaddr */
#define CONFIG_ENV_OVERWRITE

#define CONFIG_BAUDRATE         38400

#ifndef CONFIG_COMMANDS
#define CONFIG_COMMANDS		CONFIG_CMD_DFL
#endif

//#define CONFIG_COMMANDS		(CONFIG_CMD_DFL	| CFG_CMD_PCMCIA | CFG_CMD_IDE)

/* this must be included AFTER the definition of CONFIG_COMMANDS (if any) */
#include <cmd_confdefs.h>

//#define CONFIG_BOOTDELAY        3
//#define CONFIG_BOOTARGS         "root=/dev/mtdblock1 rw console=ttyS0,38400"
//#define CONFIG_ETHADDR          08:00:3e:26:0a:5b
//#define CONFIG_ETH2ADDR		08:00:3e:26:0a:5c
//#define CONFIG_NETMASK          255.255.255.0
//#define CONFIG_IPADDR           192.168.1.160
//#define CONFIG_SERVERIP         192.168.1.76
//#define CONFIG_BOOTCOMMAND      "FIXME"

#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CONFIG_KGDB_BAUDRATE    230400          /* speed to run kgdb serial port */
#define CONFIG_KGDB_SER_INDEX   2               /* which serial port to use */
#endif

/*
 * Miscellaneous configurable options
 */
//#define CFG_LONGHELP                            /* undef to save memory         */
#define CFG_PROMPT              "ArmBoot> "   /* Monitor Command Prompt       */
#define CFG_CBSIZE              256             /* Console I/O Buffer Size      */
#define CFG_PBSIZE (CFG_CBSIZE+sizeof(CFG_PROMPT)+16) /* Print Buffer Size */
#define CFG_MAXARGS             16              /* max number of command args   */
#define CFG_BARGSIZE            CFG_CBSIZE      /* Boot Argument Buffer Size    */

#undef  CFG_CLKS_IN_HZ          /* everything, incl board info, in Hz */

#define CFG_HZ                  3686400         /* incrementer freq: 3.6864 MHz */
#define CFG_CPUSPEED            CLOCK_200MHz    /* set core clock to 200/300/400 MHz */
#define CFG_CPUSPEED_TURBO	0		/* 0/1 switch to turbo mode clock */

                                                /* valid baudrates */
#define CFG_BAUDRATE_TABLE      { 9600, 19200, 38400, 57600, 115200 }

#ifndef __ASSEMBLY__
/*
 * Board specific extension for bd_info
 *
 * This structure is embedded in the global bd_info (bd_t) structure
 * and can be used by the board specific code (eg board/...)
 */

struct bd_info_ext
{
};
#endif

/*
 * Stack sizes
 *
 * The stack sizes are set up in start.S using the settings below
 */
#define CONFIG_STACKSIZE        (128*1024)      /* regular stack */
#ifdef CONFIG_USE_IRQ
#define CONFIG_STACKSIZE_IRQ    (4*1024)        /* IRQ stack */
#define CONFIG_STACKSIZE_FIQ    (4*1024)        /* FIQ stack */
#endif

/*
 * Physical Memory Map
 */
#define CONFIG_NR_DRAM_BANKS    4          /* we have 2 banks of DRAM */
#define PHYS_SDRAM_1            0xa0000000 /* SDRAM Bank #1 */
#define PHYS_SDRAM_2            0xa4000000 /* SDRAM Bank #2 */
#define PHYS_SDRAM_3            0xa8000000 /* SDRAM Bank #3 */
#define PHYS_SDRAM_4            0xac000000 /* SDRAM Bank #4 */

#define PHYS_FLASH_1            0x00000000 /* Flash Bank #1 */
#define PHYS_FLASH_2            0x04000000 /* Flash Bank #1 */
#define PHYS_FLASH_SIZE         0x02000000 /* 32 MB */

#define CFG_DRAM_BASE           PHYS_SDRAM_1

#define CFG_FLASH_BASE          PHYS_FLASH_1

#define CFG_LOAD_ADDR           PHYS_SDRAM_1 + 0x300000     /* default load address */

#define CFG_MEMTEST_START       CFG_DRAM_BASE      /* memtest works on     */
#define CFG_MEMTEST_WORDS       0x10000      /* 64 kW in DRAM   */

/*
 * GPIO settings
 */
#ifdef CONFIG_TRITON
// GPIOs for TRITON Starter Kit
#define CFG_GPSR0_VAL       0x01800000	// GPIO pin output set registers
#define CFG_GPSR1_VAL       0x70FF0000    
#define CFG_GPSR2_VAL       0x00000000

#define CFG_GPCR0_VAL       0xFE7FFFFF	// GPIO pin output clear registers
#define CFG_GPCR1_VAL       0x8F00FFFF    
#define CFG_GPCR2_VAL       0xFFFFFFFF

#define CFG_GPDR0_VAL       0x01800800	// GPIO pin direction registers
#define CFG_GPDR1_VAL       0x7CFF0280    
#define CFG_GPDR2_VAL       0x00000000    

#define CFG_GAFR0_L_VAL     0x00000000	// GPIO alternate function registers
#define CFG_GAFR0_U_VAL     0x00012814
#define CFG_GAFR1_L_VAL     0x00988050
#define CFG_GAFR1_U_VAL     0x0005AAAA
#define CFG_GAFR2_L_VAL     0x00000000
#define CFG_GAFR2_U_VAL     0x00000000

#elif defined(CONFIG_CSB226)
// GPIOs for COGENT CSB226
#define CFG_GPSR0_VAL       0xffffffff  // GPIO pin output set registers
#define CFG_GPSR1_VAL       0xffffffff
#define CFG_GPSR2_VAL       0xffffffff

#define CFG_GPCR0_VAL       0x08022080  // GPIO pin output clear registers
#define CFG_GPCR1_VAL       0x00000000
#define CFG_GPCR2_VAL       0x00000000

#define CFG_GPDR0_VAL       0xcb82a8d8  // GPIO pin direction registers
#define CFG_GPDR1_VAL       0xfcffab83
#define CFG_GPDR2_VAL       0x0001ffff

#define CFG_GAFR0_L_VAL     0x80000000  // GPIO alternate function registers
#define CFG_GAFR0_U_VAL     0xa51a8010
#define CFG_GAFR1_L_VAL     0x599a9558
#define CFG_GAFR1_U_VAL     0xaaa5aaaa
#define CFG_GAFR2_L_VAL     0xaaaaaaaa
#define CFG_GAFR2_U_VAL     0x00000002

#elif defined(CONFIG_DIMMPC)
// GPIOs for Voipac dimmPC module
#define CFG_GPSR0_VAL       0xffffffff  // GPIO pin output set registers
#define CFG_GPSR1_VAL       0xffffffff
#define CFG_GPSR2_VAL       0xffffffff

#define CFG_GPCR0_VAL       0x08022080  // GPIO pin output clear registers
#define CFG_GPCR1_VAL       0x00000000
#define CFG_GPCR2_VAL       0x00000000

#define CFG_GPDR0_VAL       0xc00220d8  // GPIO pin direction registers
#define CFG_GPDR1_VAL       0xfcffab82
#define CFG_GPDR2_VAL       0x0000ffff

#define CFG_GAFR0_L_VAL     0x00000000  // GPIO alternate function registers
#define CFG_GAFR0_U_VAL     0xa5000010
#define CFG_GAFR1_L_VAL     0x599a9550
#define CFG_GAFR1_U_VAL     0xaaa5aaaa
#define CFG_GAFR2_L_VAL     0xaaa00aaa
#define CFG_GAFR2_U_VAL     0x00000000

#elif defined(CONFIG_EM319)
// GPIOs for EMtest EM319 board
#define CFG_GPSR0_VAL       0xffffffff  // GPIO pin output set registers
#define CFG_GPSR1_VAL       0xffffffff
#define CFG_GPSR2_VAL       0xffffffff

#define CFG_GPCR0_VAL       0x08022080  // GPIO pin output clear registers
#define CFG_GPCR1_VAL       0x00000000
#define CFG_GPCR2_VAL       0x00000000

#define CFG_GPDR0_VAL       0xcb82a8d8  // GPIO pin direction registers
#define CFG_GPDR1_VAL       0xfcffab83
#define CFG_GPDR2_VAL       0x0001ffff

#define CFG_GAFR0_L_VAL     0x80000000  // GPIO alternate function registers
#define CFG_GAFR0_U_VAL     0xa51a8010
#define CFG_GAFR1_L_VAL     0x699a9550
#define CFG_GAFR1_U_VAL     0xaaa5aaaa
#define CFG_GAFR2_L_VAL     0x00050000
#define CFG_GAFR2_U_VAL     0x00000002

#else
#error "GPIO settings missing for specified target platform"
#endif

#define CFG_PSSR_VAL        0x20

/*
 * Memory settings
 */
#ifdef CONFIG_TRITON
// Memory config  for TRITON Starter Kit
#define CFG_MSC0_VAL        0x7FF024D0 
#define CFG_MSC1_VAL        0x7FF07FF0
#define CFG_MSC2_VAL        0x7FF07FF0

#elif defined(CONFIG_CSB226)
// Memory config  for COGENT CSB226
#define CFG_MSC0_VAL        0x2ef15af0
#define CFG_MSC1_VAL        0x32c432c4
#define CFG_MSC2_VAL        0x7ff17ff1

#elif defined(CONFIG_DIMMPC)
// Memory config  for Voipac dimmPC module
#define CFG_MSC0_VAL        0x2ef15af0
#define CFG_MSC1_VAL        0x32cc32cc
#define CFG_MSC2_VAL        0x3ff43ff4

#elif defined(CONFIG_EM319)
// Memory config  for EMtest EM319 board
#define CFG_MSC0_VAL        0x7ffc5af0
#define CFG_MSC1_VAL        0x3ff43ff4
#define CFG_MSC2_VAL        0x7ff47ff4

#else
#error "MSC settings missing for specified target platform"
#endif

//#define CFG_MDCNFG_VAL      0x00001AC9              
//#define CFG_MDREFR_VAL      0x000BC018             
//#define CFG_MDREFR_VAL_100  0x00018018            
//#define CFG_MDMRS_VAL       0x00000000           

/* 
 * PCMCIA and CF Interfaces 
 */

#ifdef CONFIG_TRITON
// PCMCIA config  for TRITON Starter Kit
#define CFG_MECR_VAL        0x00000003  
#define CFG_MCMEM0_VAL      0x00008086 
#define CFG_MCMEM1_VAL      0x00008086
#define CFG_MCATT0_VAL      0x00008086
#define CFG_MCATT1_VAL      0x00008086
#define CFG_MCIO0_VAL       0x00008086
#define CFG_MCIO1_VAL       0x00008086

#elif defined(CONFIG_CSB226)
// PCMCIA config  for COGENT CSB226
#define CFG_MECR_VAL        0x00000000
#define CFG_MCMEM0_VAL      0x00010504
#define CFG_MCMEM1_VAL      0x00010504
#define CFG_MCATT0_VAL      0x00010504
#define CFG_MCATT1_VAL      0x00010504
#define CFG_MCIO0_VAL       0x00004715
#define CFG_MCIO1_VAL       0x00004715

#elif defined(CONFIG_DIMMPC)
// PCMCIA config  for Voipac dimmPC module
#define CFG_MECR_VAL        0x00000000
#define CFG_MCMEM0_VAL      0x00010504
#define CFG_MCMEM1_VAL      0x00010504
#define CFG_MCATT0_VAL      0x00010504
#define CFG_MCATT1_VAL      0x00010504
#define CFG_MCIO0_VAL       0x000fcfff
#define CFG_MCIO1_VAL       0x000fcfff

#elif defined(CONFIG_EM319)
// PCMCIA config  for EMtest EM319 board
#define CFG_MECR_VAL        0x00000000
#define CFG_MCMEM0_VAL      0x00010504
#define CFG_MCMEM1_VAL      0x00010504
#define CFG_MCATT0_VAL      0x00010504
#define CFG_MCATT1_VAL      0x00010504
#define CFG_MCIO0_VAL       0x00004715
#define CFG_MCIO1_VAL       0x00004715

#else
#error "PCMCIA settings missing for specified target platform"
#endif

//#define _LED        0x08000010
//#define LED_BLANK  (0x08000040) 

/*
 * FLASH and environment organization
 */
#define CFG_FLASH_DEVS		2	/* number of flash devs in parallel */
#define CFG_MAX_FLASH_BANKS     1       /* max number of memory banks           */
#define CFG_MAX_FLASH_SECT      128  /* max number of sectors on one chip    */

/* timeout values are in ticks */
#define CFG_FLASH_ERASE_TOUT    (5*CFG_HZ) /* Timeout for Flash Erase */
#define CFG_FLASH_WRITE_TOUT    (3*CFG_HZ) /* Timeout for Flash Write */

/* FIXME */
#define CFG_ENV_ADDR            (PHYS_FLASH_1 + 0x1C000)        /* Addr of Environment Sector   */
#define CFG_ENV_SIZE            0x4000  /* Total Size of Environment Sector     */

#if (CONFIG_COMMANDS & CFG_CMD_PCMCIA)
 /*-----------------------------------------------------------------------
  * PCMCIA stuff
  *-----------------------------------------------------------------------
  */
/* we pick the upper one */

#define CONFIG_PCMCIA_SLOT_A

#define CFG_PCMCIA_IO_ADDR	(0x20000000)
#define CFG_PCMCIA_IO_SIZE	( 64 << 20 )
#define CFG_PCMCIA_DMA_ADDR	(0x24000000)
#define CFG_PCMCIA_DMA_SIZE	( 64 << 20 )
#define CFG_PCMCIA_ATTRB_ADDR	(0x2C000000)
#define CFG_PCMCIA_ATTRB_SIZE	( 64 << 20 )
#define CFG_PCMCIA_MEM_ADDR	(0x28000000)
#define CFG_PCMCIA_MEM_SIZE	( 64 << 20 )

/* in fact, MEM and ATTRB are swapped - has to be corrected soon in cmd_pcmcia or so */
#endif

#if (CONFIG_COMMANDS & CFG_CMD_IDE)
/*-----------------------------------------------------------------------
 *  * IDE/ATA stuff (Supports IDE harddisk on PCMCIA Adapter)
 *   *-----------------------------------------------------------------------
 *    */
#define CONFIG_DOS_PARTITION	1		/* DOS partitiion support */

//#define	CONFIG_IDE_PCCARD	1	/* Use IDE with PC Card	Adapter	*/

#undef	CONFIG_IDE_PCMCIA		/* Direct IDE    not supported	*/
#undef	CONFIG_IDE_LED			/* LED   for ide not supported	*/
#undef	CONFIG_IDE_RESET		/* reset for ide not supported	*/

#define CFG_IDE_MAXBUS		1	/* max. 1 IDE bus		*/
#define CFG_IDE_MAXDEVICE	1	/* max. 1 drive per IDE bus	*/

#define CFG_ATA_IDE0_OFFSET	0x0000

/* it's simple, all regs are in I/O space */
//#define CFG_ATA_BASE_ADDR	CFG_PCMCIA_ATTRB_ADDR
#define CFG_ATA_BASE_ADDR       CFG_PCMCIA_IO_ADDR

/* Offset for data I/O			*/
#define CFG_ATA_DATA_OFFSET	(CFG_ATA_BASE_ADDR)

/* Offset for normal register accesses	*/
#define CFG_ATA_REG_OFFSET	(CFG_ATA_BASE_ADDR)

/* Offset for alternate registers	*/
#define CFG_ATA_ALT_OFFSET	(CFG_ATA_BASE_ADDR)

#endif

#endif  /* __CONFIG_H */
