/*
 * Very simple (yet, for some reason, very effective) memory tester.
 * Originally by Simon Kirby <sim@stormix.com> <sim@neato.org>
 * Version 2 by Charles Cazabon <memtest@discworld.dyndns.org>
 *
 * This file contains the function declarations for the actual tests.
 * See ABOUT, CHANGELOG, and the manpage for details.
 *
 */

#ifndef _MEMTEST_H
#define _MEMTEST_H

/* Defines. */
#define OK	0
#define ERROR	-1
#define RAND32		(ui32) ( (ui32) rand () | ( (ui32) rand () << 16))
#define MAKE32FROM8(X)	(ui32) (X | X << 8 | X << 16 | X << 24)

typedef unsigned char   ui8;
typedef unsigned long   ui32;

/* Datatypes. */
typedef struct teststruct_noarg
{
	char *name;
	int (*fp)(ui32*, ui32*, ui32);
} teststruct_0;

typedef struct teststruct_onearg
{
	char *name;
	int (*fp)(ui32*, ui32*, ui32, int);
	int arg;
} teststruct_1;

extern teststruct_0 tests_noarg[];
extern teststruct_1 tests_1arg[];

/* Function declarations. */
int test_verify_success (ui32 *bp1, ui32 *bp2, ui32 count);
int test_random_value (ui32 *bp1, ui32 *bp2, ui32 count);
int test_xor_comparison (ui32 *bp1, ui32 *bp2, ui32 count);
int test_sub_comparison (ui32 *bp1, ui32 *bp2, ui32 count);
int test_mul_comparison (ui32 *bp1, ui32 *bp2, ui32 count);
int test_div_comparison (ui32 *bp1, ui32 *bp2, ui32 count);
int test_or_comparison (ui32 *bp1, ui32 *bp2, ui32 count);
int test_and_comparison (ui32 *bp1, ui32 *bp2, ui32 count);
int test_seqinc_comparison (ui32 *bp1, ui32 *bp2, ui32 count);
int test_checkerboard_comparison (ui32 *bp1, ui32 *bp2, ui32 count);
int test_solidbits_comparison (ui32 *bp1, ui32 *bp2, ui32 count);
int test_blockseq_comparison (ui32 *bp1, ui32 *bp2, ui32 count);
int test_walkbits_comparison (ui32 *bp1, ui32 *bp2, ui32 count, int mode);
int test_bitspread_comparison (ui32 *bp1, ui32 *bp2, ui32 count);
int test_bitflip_comparison (ui32 *bp1, ui32 *bp2, ui32 count);
int test_stuck_address (ui32 *bp1, ui32 *bp2, ui32 count);

extern int rand(void);

#endif /* _MEMTEST_H */

