/*
 * (C) Copyright 2000
 *
 * See file CREDITS for list of people who contributed to this project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <armboot.h>
#include <command.h>
#include "net.h"
#include "icmp.h"

#if (CONFIG_COMMANDS & CFG_CMD_NET)

#define TIMEOUT		1		/* Seconds before trying ICMP again */

static void IcmpEchoHandler(uchar *pkt, unsigned dest, unsigned src, unsigned len);
static void IcmpEchoTimeout(void);

ushort icmp_seq = 0;
ushort icmp_received = 0;

void IcmpEchoPrintStats(void)
{
	printf("\n --- ");
	print_IPaddr( NetServerIP);
	printf(" ping statistics ---\n");
	printf(" %u packets transmitted, %u packets received", icmp_seq, icmp_received);

	if( icmp_seq > 0)
		printf(", %u%% packet loss", 100 - (icmp_received*100)/icmp_seq);
	putc('\n');
}

void IcmpArpHandler(uchar *pkt, unsigned dest, unsigned src, unsigned len)
{
	if( icmp_seq == 0)
	{
		printf("Pinging ");
		print_IPaddr( NetServerIP);
		printf(" with %u bytes of data:\n", ICMP_ECHO_REQUEST_SIZE);
	
		IcmpRequest(ICMP_ECHO_REQUEST);
	}
}

/*
 *	Handle a ICMP received packet.
 */
static void IcmpEchoHandler(uchar *pkt, unsigned dest, unsigned src, unsigned len)
{
	IP_t* iph = (IP_t*) pkt;
	ICMP_t* icmph = (ICMP_t*) ((uchar*)iph + IP_HDR_SIZE_NO_UDP);

	if( icmph->un.echo.id != ICMP_ECHO_REQUEST_SIZE)
		return;
	
	switch( icmph->type)
	{
		case ICMP_ECHO_REPLY:
		{
			printf (" %u bytes from ", len - IP_HDR_SIZE_NO_UDP - sizeof(ICMP_t));
			print_IPaddr (src);
			printf (": icmp_seq=%u ttl=%u\n", icmph->un.echo.sequence, iph->ip_ttl);
			icmp_received++;
		} break;
		
		case ICMP_DEST_UNREACHABLE:
		{
			printf(" Destination ");
			switch( icmph->code)
			{
				case 0:	// net unreachable
					printf("net "); break;
				case 1:	// host unreachable
					printf("host "); break;
				case 2:	// protocol unreachable
					printf("protocol "); break;
				case 3:	// port unreachable
					printf("port "); break;
			}
			printf("unreachable.\n");
		} break;
		
		case ICMP_SOURCE_QUENCH:
			printf(" Source quench received.\n"); break;
			
		case ICMP_TIME_EXCEEDED:
			printf(" Request timed out.\n"); break;

		default:
			break;
	}			
}


/*
 *	Timeout on ICMP request.  Try again, forever.
 */
static void IcmpEchoTimeout(void)
{
	IcmpRequest (ICMP_ECHO_REQUEST);
}


void IcmpRequest (int type)
{
	int i, len = 0;
	ICMP_t*	icmph = (ICMP_t*) (NetTxPacket + ETHER_HDR_SIZE + IP_HDR_SIZE_NO_UDP);
	volatile uchar* data = (uchar*)(NetTxPacket + ETHER_HDR_SIZE + IP_HDR_SIZE_NO_UDP + sizeof(ICMP_t));

	NetSetEther (NetTxPacket, NetServerEther, PROT_IP);
	
	switch( type)
	{
		case ICMP_ECHO_REQUEST:
		{
			icmph->type = ICMP_ECHO_REQUEST;
			icmph->code = 0;
			icmph->checksum = 0;
			icmph->un.echo.id = ICMP_ECHO_REQUEST_SIZE;
			icmph->un.echo.sequence = icmp_seq++;

			for( i=0; i<ICMP_ECHO_REQUEST_SIZE;i++)
				data[i] = i;
		
			len = sizeof(ICMP_t) + ICMP_ECHO_REQUEST_SIZE; 
			
			icmph->checksum = ~NetCksum((uchar *)icmph, len>>1);
			
			NetSetHandler(IcmpEchoHandler);
			NetSetTimeout(TIMEOUT * CFG_HZ, IcmpEchoTimeout);
		} break;

		default:
			return;
	}
	
	NetSetIP( NetTxPacket + ETHER_HDR_SIZE, NetServerIP, IPPROTO_ICMP, len);	
	NetSendPacket(NetTxPacket, ETHER_HDR_SIZE + IP_HDR_SIZE_NO_UDP + len);
}

#endif /* CFG_CMD_NET */

